import GFFFile
import ERFFile
import NeverData

class Area (NeverData.NeverData):
    areFile = None
    gitFile = None
    gicFile = None
    name = ''

    arePropList = {
        'Name': 'CExoLocString',
        'ChanceLightning': 'Percentage',
        'ChanceRain': 'Percentage',
        'ChanceSnow': 'Percentage',
        'DayNightCycle': 'Boolean',
        'IsNight': 'Boolean',
        'ModListenCheck': 'Integer',
        'ModSpotCheck': 'Integer',
        'MoonFogAmount': 'Integer,0-15',
        'MoonShadows': 'Boolean',
        'NoRest': 'Boolean',
        'ShadowOpacity': 'Percentage',
        'SunFogAmount': 'Integer,0-15',
        'SunShadows': 'Boolean',
        'WindPower': 'Integer,0-2',
        'OnEnter': 'ResRef,NSS',
        'OnExit': 'ResRef,NSS',
        'OnHeartbeat': 'ResRef,NSS',
        'OnUserDefined': 'ResRef,NSS',
        'MoonAmbientColor': 'BGRColour',
        'MoonDiffuseColor': 'BGRColour',
        'SunDiffuseColor': 'BGRColour',
        'SunDiffuseColor': 'BGRColour',
        'SunFogColor': 'BGRColour',
        'MoonFogColor': 'BGRColour'
        #'Tileset': 'ResRef,SET' #not handling resrefs not in mod yet
        }

    gitPropList = {
        'AreaProperties.AmbientSndDayVol': 'Integer,0-127',
        'AreaProperties.AmbientSndNitVol': 'Integer,0-127'
        }
    
    def __init__(self,erfFile,areaName):
        self.name = areaName
        area = erfFile.getEntryByNameAndExtension(areaName,'ARE')
        if not area:
            print "couldn't find area for ",areaName
            return None
        git = erfFile.getEntryByNameAndExtension(areaName,'GIT')
        gic = erfFile.getEntryByNameAndExtension(areaName,'GIC')
        
        self.areFile = erfFile.getEntryContents(area)
        self.gitFile = erfFile.getEntryContents(git)
        self.gicFile = erfFile.getEntryContents(gic)

    def getProperty(self,label):        
        if label in self.arePropList:
            return [label,\
                    self.areFile.getRoot().getInterpetedEntry(label),\
                    self.arePropList[label]]

    def setProperty(self,label,value):
        if label in self.arePropList:
            (s,t) = self.getTargetStructAndLabel(self.areFile,label)
            s.setInterpretedEntry(t,value)
        elif label in self.gitPropList:
            (s,t) = self.getTargetStructAndLabel(self.gitFile,label)
            s.setInterpretedEntry(t,value)
        else:
            print "error in Area.py: can't find prop",label,value
            
    def iterateProperties(self):
        pList = self.arePropList.keys()
        pList.sort()
        for label in pList:
            (s,t) = self.getTargetStructAndLabel(self.areFile,label)
            yield [label,\
                   s.getInterpretedEntry(t),\
                   self.arePropList[label]]
        pList = self.gitPropList.keys()
        pList.sort()
        for label in pList:
            (s,t) = self.getTargetStructAndLabel(self.gitFile,label)
            yield [label,\
                   s.getInterpretedEntry(t),\
                   self.gitPropList[label]]

    def getName(self,lang=0):
        return self.areFile.getRoot().getInterpretedEntry('Name').getString()

    def getChanceLightning(self):
        return self.areFile.getRoot().getInterpretedEntry('ChanceLightning')

    def getChanceRain(self):
        return self.areFile.getRoot().getInterpretedEntry('ChanceRain')

    def getChangeSnow(self):
        return self.areFile.getRoot().getInterpretedEntry('ChanceSnow')

    def getChangeLightning(self):
        return self.areFile.getRoot().getInterpretedEntry('ChanceLightning')

    def getComments(self):
        return self.areFile.getRoot().getInterpretedEntry('Comments')

    def getDayNightCycle(self):
        return self.areFile.getRoot().getInterpretedEntry('DayNightCycle')

    def getIsExterior(self):
        flags = self.areFile.getRoot().getInterpretedEntry('Flags')
        return flags & 0x001
    
    def getIsUnderground(self):
        flags = self.areFile.getRoot().getInterpretedEntry('Flags')
        return flags & 0x002

    def getIsNatural(self):
        flags = self.areFile.getRoot().getInterpretedEntry('Flags')
        return flags & 0x004

    def getHeight(self):
        return self.areFile.getRoot().getInterpretedEntry('Height')

    def getIsNight(self):
        return self.areFile.getRoot().getInterpretedEntry('IsNight')

    def getModListenCheck(self):
        return self.areFile.getRoot().getInterpretedEntry('ModListenCheck')

    def getModSpotCheck(self):
        return self.areFile.getRoot().getInterpretedEntry('ModSpotCheck')

    def getNoRest(self):
        return self.areFile.getRoot().getInterpretedEntry('NoRest')

    def getWidth(self):
        return self.areFile.getRoot().getInterpretedEntry('Width')
    




    

    
