/**
 * Sample NWN File Viewer using libnw
 */

#include "FileViewer.h"

#define MENU_LINK(name) EVT_MENU(FileViewer_##name, FileViewer::On##name)
BEGIN_EVENT_TABLE(FileViewer, wxFrame)
    MENU_LINK(Open)
    MENU_LINK(About)
    EVT_TREE_SEL_CHANGED(FileStructureTree_Ctrl, FileViewer::OnTreeSelect)
#undef MENU_LINK
END_EVENT_TABLE()

    //BEGIN_EVENT_TABLE(FileViewer,wxTreeCtrl)
    //END_EVENT_TABLE()

IMPLEMENT_APP(FileViewerApp)

bool FileViewerApp::OnInit() {
    printf("FileViewer starting up...\n");
    nw_inp_dir = get_game_dir();
    FileViewer *frame = new FileViewer("neveredit",
				 50,50,500,400);
    frame->Show(TRUE);
    SetTopWindow(frame);
    return true;
}

FileViewer::FileViewer(const wxString& t, int x, int y, int w, int h) :
    wxFrame((wxFrame*)NULL,-1,t,wxPoint(x,y),wxSize(w,h),wxDEFAULT_FRAME_STYLE) {
    SetBackgroundColour(wxColour(255, 255, 255));
    
    wxMenu *fileMenu = new wxMenu;
    fileMenu->Append(FileViewer_Open, _T("&Open..."));

    wxMenu *helpMenu = new wxMenu;
    wxMenuItem *helpDummy = new wxMenuItem(helpMenu, 999, _T("No Help for neveredit yet"));
    helpMenu->Append(FileViewer_About, _T("&About...\tF1"), _T("Show About Dialog"));
    helpMenu->Append(helpDummy);
    helpDummy->Enable(false); //not sure why this isn't working (on mac, that is)...

    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(fileMenu, _T("&File"));
    menuBar->Append(helpMenu, _T("&Help"));

    SetMenuBar(menuBar);
    splitter = new wxSplitterWindow(this, -1);
    tree = new FileStructureTree(splitter,FileStructureTree_Ctrl);
    listCtrl = new wxListCtrl(splitter,-1,wxDefaultPosition,wxDefaultSize,wxLC_REPORT | wxSUNKEN_BORDER);
    splitter->SplitVertically(tree,listCtrl,200);

    listCtrl->InsertColumn(0,"Name", wxLIST_FORMAT_LEFT,150);
    listCtrl->InsertColumn(1,"Value",wxLIST_FORMAT_LEFT,300);

    CreateStatusBar();
    SetStatusText("Welcome to neveredit");
}

FileViewer::~FileViewer() {
    if(tree != NULL) {
	delete tree;
    }
}

void FileViewer::clearList() {
    listCtrl->DeleteAllItems();
}

void FileViewer::clearTree() {
    tree->DeleteAllItems();
}

void FileViewer::addListItem(std::string name, std::string value) {
    int index = listCtrl->InsertItem(listCtrl->GetItemCount(),name.c_str());
    listCtrl->SetItem(index,1,value.c_str());
}

void FileViewer::OnOpen(wxCommandEvent& WXUNUSED(event)) {
    SetStatusText("");
    wxString filename = wxFileSelector("Choose a mod file to open",
				       ".","",".mod",
				       "Mod Files (*.mod)|*.mod|NWN Mod Files (*.nwm)|*.nwm|All Files|*.*");
    //uncomment the next and change it to a mod file if you want to avoid
    //the file dialog
    //wxString filename = "/Users/pgorniak/src/libnw/coc/coc.mod";
    if(filename.empty()) {
	return;
    }
    clearList();
    clearTree();
    mod.loadFile(filename.c_str());
    tree->treeFromMODFile(mod);
    SetStatusText(wxString("loaded") + filename);    
}

void FileViewer::OnAbout(wxCommandEvent& WXUNUSED(event)) {
    wxMessageBox(_T("neveredit using libnw\nCopyright 2003 Openknights Consortium"), 
		 _T("About neveredit"), wxOK | wxICON_INFORMATION, this);
}

void FileViewer::OnTreeSelect(wxTreeEvent& event) {    
    clearList();
    wxString t = tree->GetItemText(event.GetItem());
    //printf("selected %s\n", t.c_str());
    TreeItemData *data = (TreeItemData*)tree->GetItemData(event.GetItem());
    if(data == NULL) {
	return;
    }
    NWNDataType item = data->getItem();
    std::map<std::string,NWNDataType>::iterator i = item.listValue.begin();   
    for(;i != item.listValue.end();i++) {
	if(i->second.type != t_list) {
	    addListItem(i->first,i->second.toString());
	}
    }
}
