#include "MODFile.h"

MODFile::MODFile() : toc(NULL) {
}

MODFile::MODFile(std::string fname, int off) : toc(NULL) {
    loadFile(fname,off);
}

void MODFile::loadFile(std::string f) {
    loadFile(f,0);
}

void MODFile::loadFile(std::string fname, int off) {
    clear();
    indexToNameMap.clear();
    modFileName = fname;
    //should free toc here if allocated, have to look at that...
    agg_toc_delete(toc);
    toc = agg_toc_load(fname.c_str(),off);
    for(int resc=0;resc<toc->head->rescnt;resc++) {
	struct nwn_agg_resc* rp = (nwn_agg_resc*)vsubig(toc->rescs, resc);
	const char* t = typ2ext(rp->type);
	char* f = 0;
	
	if (strcmp(t, "???") == 0) {
	    f = new char[9];
	    sprintf(f, "%.8lx", rp->type);
	    fprintf(stderr, "neveredit: Warning: Unknown resource type (%s)!\n",f);
	} else {
	    f = strdup(t);
	}
        int len = 0;
        int j = 0;
        char *s;
        for (s = rp->name, j = 0; j < 16 && s[j]; ++j)
            ;
        len = j;

        std::string name(s,len);
	std::pair<std::string,std::string> p(name,f);
	(*this)[p] = resc;
        //printf("mapping %d to \"%s.%s\"\n",resc,p.first.c_str(),p.second.c_str());
        //printf("lengths are %d and %d\n", p.first.size(), p.second.size());
	indexToNameMap[resc] = p;
	delete f;
    }
}

std::pair<std::string,std::string> MODFile::getNameAndType(int index) const {
    std::map<int,std::pair<std::string,std::string> >::const_iterator i = 
	indexToNameMap.find(index);
    if(i == indexToNameMap.end()) {
	return std::pair<std::string,std::string>("","");
    } else {
	return i->second;
    }
}
    
std::string MODFile::getName(int index) const {
    return getNameAndType(index).first;
}
    
std::string MODFile::getType(int index) const {
    return getNameAndType(index).second;
}

int MODFile::getFileOffset(std::string name, std::string ext) const {
    const_iterator i = find(std::pair<std::string,std::string>(name,ext));
    if(i == end()) {
        printf("couldn't find offset for \"%s.%s\"\n",name.c_str(),ext.c_str());
	return -1;
    } else {
	return getFileOffset(i->second);
    }
}

int MODFile::getFileOffset(int index) const {
    struct nwn_agg_so* sp = (nwn_agg_so*)vsubig(toc->szofs,index);
    return sp->off;
}

ITPResource& MODFile::getITPResource(int index) {
    std::pair<std::string,std::string> nameAndType = getNameAndType(index);
    return getITPResource(nameAndType.first,nameAndType.second);
}
    
ITPResource& MODFile::getITPResource(std::string name, std::string ext) {
    std::map<std::string,ITPResource>::iterator i = itpResources[ext].find(name);
    if(i != itpResources[ext].end()) {
	i->second;
    } else {
	int offset = getFileOffset(name,ext);
	if(offset < 0) {
	    return itpResources[""][""]; //probably throw exception instead?
	} else {
	    return itpResources[ext][name].fromFile(modFileName,offset);
	}
    }
}
